from .chat import ChatAsync, Chat
from .image import ImageAsync, Image
from .tts import TTSAsync, TTS


class ShardClient:
    def __init__(self, api_key: str):
        self.api_key = api_key
        self.chat = Chat(api_key)
        self.chat_async = ChatAsync(api_key)
        self.image = Image(api_key)
        self.image_async = ImageAsync(api_key)
        self.tts = TTS(api_key)
        self.tts_async = TTSAsync(api_key)

    def __repr__(self):
        return f"<ShardClient api_key={self.api_key}>"

    def __str__(self):
        return f"<ShardClient api_key={self.api_key}>"

    def __eq__(self, other):
        return self.api_key == other

    def __ne__(self, other):
        return self.api_key != other

    def __hash__(self):
        return hash(self.api_key)

    def __bool__(self):
        return bool(self.api_key)

    def __len__(self):
        return len(self.api_key)



api_key = ""

from shardai import ShardClient

client = ShardClient(api_key)


async def test_tts():
    prompt = "Hello, how are you?"
    voice = "Rachel"

    response = await client.tts_async.completions(prompt, voice)
    print(response.audio)


async def test_tts_voices():
    print(await client.tts_async.voices())


async def test_chat():
    print(await client.chat_async.models())
    prompt = "Hello, how are you?"
    response = await client.chat_async.completions(prompt, "llama_2_7b")
    print(response)
    print(response.choices)
    print(response.choices[0].message.content)
    print(response.choices[0].message.role)


async def test_image():
    prompt = "A beautiful sunset"
    response = await client.image_async.completions(prompt)
    print(response.image)
    print(response.generation_time)
    print(response.warning)


async def test_image_models():
    print(await client.image_async.options())


async def test_all():
    await test_tts()
    await test_tts_voices()
    await test_chat()
    await test_image()
    await test_image_models()


def main():
    import asyncio

    loop = asyncio.get_event_loop()
    loop.run_until_complete(test_all())


if __name__ == "__main__":
    main()



from .objects import ImageResponse, ImageOptions
from requests import get
from aiohttp import ClientSession
from random import randint
from .exceptions import *


class ImageAsync:
    def __init__(self, api_key: str):
        self.api_key = api_key

    async def completions(
        self,
        prompt: str = None,
        sampler: str = "DPM++ SDE Karras",
        ratio: str = "square",
        model: str = "DREAMSHAPER_8",
        cfg: int = 4,
        steps: int = 15,
        negative_prompt: str = None,
        seed: int = randint(0, 10000000000),
    ):
        """
        Image generation function
        """
        if self.api_key is None:
            raise NoAPIKeyError("API key is required for this function")
        if prompt is None:
            raise NoInputError("Prompt is required for this function")
        headers = {"api-key": self.api_key, "Content-Type": "application/json"}
        payload = {
            "prompt": prompt,
            "sampler": sampler,
            "ratio": ratio,
            "model": model,
            "cfg": cfg,
            "steps": steps,
            "negative_prompt": negative_prompt,
            "seed": seed,
        }
        async with ClientSession() as session:
            async with session.post(
                "https://shard-ai.xyz/v1/sd1x/completions",
                json=payload,
                headers=headers,
            ) as response:
                json_response = await response.json()
                if response.status != 200:
                    raise APIError(f"Error: {response.status}")
                return ImageResponse(
                    json_response["image"],
                    json_response["generation-time"],
                    json_response["Warning!"],
                )

    async def options(self):
        """
        Get available options
        """
        async with ClientSession() as session:
            async with session.get("https://shard-ai.xyz/v1/sd1x/models") as response:
                json_response = await response.json()
                if response.status != 200:
                    raise APIError(f"Error: {response.status}")
                return ImageOptions(
                    json_response["models"],
                    json_response["ratios"],
                    json_response["samplers"],
                    json_response["upscale"],
                )


class Image:
    def __init__(self, api_key: str):
        self.api_key = api_key

    def completions(
        self,
        prompt: str = None,
        sampler: str = "DPM++ SDE Karras",
        ratio: str = "square",
        model: str = "DREAMSHAPER_8",
        cfg: int = 4,
        steps: int = 15,
        negative_prompt: str = None,
        seed: int = randint(0, 10000000000),
    ):
        """
        Image generation function
        """
        if self.api_key is None:
            raise NoAPIKeyError("API key is required for this function")
        if prompt is None:
            raise NoInputError("Prompt is required for this function")
        headers = {"api-key": self.api_key, "Content-Type": "application/json"}
        payload = {
            "prompt": prompt,
            "sampler": sampler,
            "ratio": ratio,
            "model": model,
            "cfg": cfg,
            "steps": steps,
            "negative_prompt": negative_prompt,
            "seed": seed,
        }
        response = get(
            "https://shard-ai.xyz/v1/sd1x/completions", json=payload, headers=headers
        )
        if response.status_code != 200:
            raise APIError(f"Error: {response.status_code}")
        response_json = response.json()
        return ImageResponse(
            response_json["image"],
            response_json["generation-time"],
            response_json["Warning!"],
        )

    def options(self):
        """
        Get available options
        """
        response = get("https://shard-ai.xyz/v1/sd1x/models")
        if response.status_code != 200:
            raise APIError(f"Error: {response.status_code}")
        response_json = response.json()
        return ImageOptions(
            response_json["models"],
            response_json["ratios"],
            response_json["samplers"],
            response_json["upscale"],
        )



class TTSResponse:
    def __init__(self, audio):
        self.audio = audio


class Voice:
    def __init__(self, name, labels, preview_url):
        self.name = name
        self.age = labels.get("age")
        self.accent = labels.get("accent")
        self.description = labels.get("description")
        self.gender = labels.get("gender")
        self.use_case = labels.get("use_case")
        self.preview_url = preview_url


class ChatResponse:
    def __init__(self, id, object, created, model, usage, choices):
        self.id = id
        self.object = object
        self.created = created
        self.model = model
        self.usage = usage
        self.choices = [
            ChoiceChat(
                choices[0]["message"], choices[0]["finish_reason"], choices[0]["index"]
            )
        ]


class ChoiceChat:
    def __init__(self, message: dict, finish_reason, index):
        self.message = ChatMessage(message["role"], message["content"])
        self.finish_reason = finish_reason
        self.index = index


class ChatMessage:
    def __init__(self, role, content):
        self.role = role
        self.content = content


class ImageResponse:
    def __init__(self, image, generation_time, warning):
        self.image = image
        self.generation_time = generation_time
        self.warning = warning


class ImageOptions:
    def __init__(self, models, ratios, samplers, upscale):
        self.models = models
        self.ratios = ratios
        self.samplers = samplers
        self.upscale = upscale



from requests import get
from aiohttp import ClientSession
from .objects import TTSResponse, Voice
from .exceptions import *


class TTSAsync:
    def __init__(self, api_key: str):
        self.api_key = api_key

    async def completions(self, prompt: str, voice: str = "Rachel"):
        """
        Text to speech function
        """
        if self.api_key is None:
            raise NoAPIKeyError("API key is required for this function")
        if prompt is None:
            raise NoInputError("Prompt is required for this function")

        headers = {"api-key": self.api_key}
        payload = {"prompt": prompt, "voice": voice}
        async with ClientSession() as session:
            async with session.post(
                "https://shard-ai.xyz/v1/tts/completions", json=payload, headers=headers
            ) as response:
                json_response = await response.json()
                if response.status != 200:
                    raise APIError(f"Error: {response.status}")
                return TTSResponse(json_response["audio"])

    async def voices(self):
        """
        Get available voices
        """
        async with ClientSession() as session:
            async with session.get("https://shard-ai.xyz/v1/tts/voices") as response:
                json_response = await response.json()
                if response.status != 200:
                    raise APIError(f"Error: {response.status}")
                return [
                    Voice(voice["name"], voice["labels"], voice["preview_url"])
                    for voice in json_response["voices"]
                ]


class TTS:
    def __init__(self, api_key: str):
        self.api_key = api_key

    def completions(self, prompt: str, voice: str = "Rachel"):
        """
        Text to speech function
        """
        if self.api_key is None:
            raise NoAPIKeyError("API key is required for this function")
        if prompt is None:
            raise NoInputError("Prompt is required for this function")
        headers = {"api-key": f"{self.api_key}"}
        payload = {"prompt": prompt, "voice": voice}
        response = get(
            "https://shard-ai.xyz/v1/tts/completions", json=payload, headers=headers
        )
        if response.status_code != 200:
            raise APIError(f"Error: {response.status_code}")
        response_json = response.json()
        return TTSResponse(response_json["audio"])

    def voices(self):
        """
        Get available voices
        """
        response = get("https://shard-ai.xyz/v1/tts/voices")
        if response.status_code != 200:
            raise APIError(f"Error: {response.status_code}")
        response_json = response.json()
        return [
            Voice(voice["name"], voice["labels"], voice["preview_url"])
            for voice in response_json["voices"]
        ]



from requests import get
from aiohttp import ClientSession
from .objects import ChatResponse
from .exceptions import *


class ChatAsync:
    def __init__(self, api_key: str):
        self.api_key = api_key

    async def completions(self, prompt: str, model: str):
        """
        Chat function
        """
        if self.api_key is None:
            raise NoAPIKeyError("API key is required for this function")
        if prompt is None:
            raise NoInputError("Prompt is required for this function")

        headers = {"api-key": self.api_key, "Content-Type": "application/json"}
        payload = {"messages": [{"role": "user", "content": prompt}], "model": model}
        async with ClientSession() as session:
            async with session.post(
                "https://shard-ai.xyz/v1/chat/completions",
                json=payload,
                headers=headers,
            ) as response:
                json_response = await response.json()
                if response.status != 200:
                    raise APIError(f"Error: {response.status}")
                return ChatResponse(
                    json_response["id"],
                    json_response["object"],
                    json_response["created"],
                    json_response["model"],
                    json_response["usage"],
                    json_response["choices"],
                )

    async def models(self):
        """
        Get available models
        """
        async with ClientSession() as session:
            async with session.get("https://shard-ai.xyz/v1/chat/models") as response:
                json_response = await response.json()
                if response.status != 200:
                    raise APIError(f"Error: {response.status}")
                return json_response["models"]


class Chat:
    def __init__(self, api_key: str):
        self.api_key = api_key

    def completions(self, prompt: str, model: str):
        """
        Chat function
        """
        if self.api_key is None:
            raise NoAPIKeyError("API key is required for this function")
        if prompt is None:
            raise NoInputError("Prompt is required for this function")
        headers = {"api-key": self.api_key, "Content-Type": "application/json"}
        payload = {"messages": [{"role": "user", "content": prompt}], "model": model}
        response = get(
            "https://shard-ai.xyz/v1/chat/completions", json=payload, headers=headers
        )
        if response.status_code != 200:
            raise APIError(f"Error: {response.status_code}")
        response_json = response.json()
        return ChatResponse(
            response_json["id"],
            response_json["object"],
            response_json["created"],
            response_json["model"],
            response_json["usage"],
            response_json["choices"],
        )

    def models(self):
        """
        Get available models
        """
        response = get("https://shard-ai.xyz/v1/chat/models")
        if response.status_code != 200:
            raise APIError(f"Error: {response.status_code}")
        response_json = response.json()
        return response_json["models"]



class InvalidAPIKeyError(Exception):
    def __init__(self, error):
        self.error = error


class NoAPIKeyError(Exception):
    def __init__(self, error):
        self.error = error


class APIError(Exception):
    def __init__(self, error):
        self.error = error


class NoInputError(Exception):
    def __init__(self, error):
        self.error = error
